/*
 * MyCanvasOracle.java
 *
 * Created on 24 de marzo de 2005, 12:58
 */

package learningbom;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import oracle.paint.PaintFactorOracle;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import javax.imageio.ImageIO;
import oracle.FactorOracle;
import oracle.Transition;
import oracle.NodeOracle;
import java.awt.Cursor;
import java.util.ArrayList;
import javax.swing.JPanel;
import oracle.FactorOracleConst;


/**
 *
 * @author Alejandro Touza Paredes
 */
public class MyCanvasOracle extends MyCanvas{
    
    /** Grafo Oracle que se mostrar en el Canvas. */
    protected FactorOracle fo;
    /** Parmetros con los que se mostrar el Grafo Oracle. */
    protected PaintFactorOracle pfo;
    /** Indica la transicin actual seleccionada. */
    protected Transition currentTransition;
    /** Indica si la transicin actual es nueva o ya exista. */
    protected boolean currentTransitionIsNew;
    /** Indica el nodo actual que estamos explorando. */
    protected NodeOracle currentNodeOracle;
    /** Indica el numero de pasos realizados hasta el momento. Si se usa complete indicar
      * el nmero de pasos necesarios para completar el grafo. */
    protected int numberOfSteps;
    /** Indica que si hemos de dibujar los suffix-link del grafo. */
    protected boolean drawLink;
    /** Indica si el botn izquierdo se ha pulsado dentro del Canvas. */
    protected boolean leftButtonClicked; 
    /** Indica si el pintado del GrafoOracle se har de acuerdo con el contenido del
     *  ArrayList transitionsList. */
    protected boolean searchMode;
    /** Transiciones seleccionadas que sern pintadas de forma diferente. */
    protected ArrayList transitionsList;
    /** Indica el ltimo punto donde se ha producido un evento de ratn con el
     *  botn izquierdo pulsado. */
    protected Point pointLeftClick;
    /** Indica si hay que llamr a fitToScreen cada vez que se haga un resize de la ventana.
     *  (solo en el caso de que usemos los listeners por defecto). */
    protected boolean fitToScreen;
    
    protected static Cursor cDrag, cSelect;
    
    //CONSTRUCTORAS
    /** Crea una nueva instancia de MyCanvasOracle
     *  @param x Coordenada x donde se situar MyCanvasOracle
     *  @param y Coordenada y donde se situar MyCanvasOracle
     *  @param d Dimensiones de MyCanvasOracle */ 
    public MyCanvasOracle(JPanel jpanel){
        
        super(jpanel);
        
        drawLink = false;
        searchMode = false;
        fitToScreen = false;
        transitionsList = new ArrayList();
        resetParameters();
        pfo = new PaintFactorOracle();
        pfo.setAutomatonStartPoint(getInitialPoint());
        initGraphics2D();
        setBackground(pfo.getBackgroundColor());
        
        //Imagen del icono del cursor de seleccin
        Image img = getToolkit().getImage(getClass().getResource("/images/icons/preparedHand.gif")); 
        //Preparamos los cursores
        cSelect = java.awt.Toolkit.getDefaultToolkit().createCustomCursor(
                        img, 
                        new Point (img.getHeight(this)/2,   //Centro del cursor
                                   img.getWidth(this)/2), 
                        "preparedHand");
        
        //Imagen del icono del cursor de arrastrado
        img = getToolkit().getImage(getClass().getResource("/images/icons/dragHand.gif")); 
        
        cDrag = java.awt.Toolkit.getDefaultToolkit().createCustomCursor(
                        img,
                        new Point (img.getHeight(this)/2,   //Centro del cursor
                                   img.getWidth(this)/2), 
                        "dragHand");
        
        this.repaint();
    }

    public boolean getFitToScreen(){
        
        return fitToScreen;
    }
    public void setFitToScreen(boolean b){
        
        fitToScreen = b;
    }
    //////////////////////////////////////////////////////////////////////////////////
    // EVENTOS DE RATN
    /** Registra si se ha producido un click del botn izquierdo y si es as 
     *  recoge el punto donde se ha producido.
     *  @param evt Evento de ratn. */
    protected void mousePressedEvent(java.awt.event.MouseEvent evt){
                
        if(start && evt.getButton()==evt.BUTTON1) //Botn Izqdo.
        {
            leftButtonClicked = true;
            pointLeftClick = evt.getPoint();
            setCursor(cDrag); //Cursor para cuando arrastremos
        }
    }
    protected void mouseReleasedEvent(java.awt.event.MouseEvent evt){
                
        if(evt.getButton()==evt.BUTTON1)
        {    
            leftButtonClicked = false; 
            if(start)
            {
                setCursor(cSelect); //Cursor normal de seleccin
                repaint();
            }
        }
    }
    /** Mueve el grafo Oracle en las direcciones marcadas por el movimiento del
     *  ratn con el botn izquierdo pulsado.
     *  @param evt Evento de ratn. */
    protected void mouseDraggedEvent(java.awt.event.MouseEvent evt){
                            
        if(leftButtonClicked)
        {
            moveFromStartPoint (evt.getPoint().x - pointLeftClick.x,
                             evt.getPoint().y - pointLeftClick.y);
            pointLeftClick = evt.getPoint();
            //paintRoughGraph();
            repaint();
        }
    }
    //////////////////////////////////////////////////////////////////////////////////
    // EVENTOS DE COMPONENTES
    /** Se encarga de hacer un resize del canvas y de adaptar el Grafo a el nuevo
     *  tamao de la ventana. */
    protected void componentResizedEvent(java.awt.event.ComponentEvent evt){
        
        super.componentResizedEvent(evt);
        initGraphics2D();
        if(fitToScreen) fitToScreen();
        repaint();
    }
    
    //////////////////////////////////////////////////////////////////////////////////
    //CONSULTORAS - MODIFICADORAS
    public boolean getLink(){
        
        return drawLink;
    }
    public boolean getSearchMode(){
        
        return searchMode;
    }
    public void setSearchMode(boolean b){
        
        searchMode = b;
    }
    public String getPattern(){
        
        return fo.pattern();
    }
    public void flushTransitionsList(){
        
        transitionsList.clear();
    }
    /** Aade una transicin dada a la lista (ojo, no se comprueban los valores de la transicin)*/
    public void addTransitionToList(Transition t){
        
        transitionsList.add(t);
    }
    /** Busca el string pasado en el Factor Oracle y aade las transiciones encontradas. 
     *  La bsqueda del String es incremental, no se busca cada carcter desde el primer
     *  estado del autmata. */
    public void addTransitionsFoundToList(String found){
    
        int node = 0;
        int nodeDesti = 0;
        String aux = "";
    
        for(int i = 0; i<found.length(); i++)
        {
            aux = found.substring(i, i+1); 
            nodeDesti = fo.getNode(node).hasTransition(aux);
            if(nodeDesti>=0)
            {
                transitionsList.add(new Transition(node, nodeDesti, aux));
                node = nodeDesti;
            }
            else return;
        }
    }       
    public void setDrawLink(boolean b){
        
        this.drawLink = b;
    }
    public void setFactorOracle(FactorOracle go){
        
        this.fo = go;
    }
    
    //FUNCIONES DE INICIALIZADO / PINTADO DEL GRAPH ORACLE
    /** Crea una instancia de Grafo Oracle con pattern. Pinta los nodos vacos 
     *  del grafo si asi se le indica.
     *  @param pattern String con el que crea una instancia de Grafo Oracle.
     *  @param b Booleano que indica si debemos pintar el Grafo tras inicializarlo.  */
    public void start(String pattern){
        
        resetParameters();
        
        start = true;
        setCursor(cSelect);
        fo = new FactorOracle(pattern);
    }
    public void reset(){
        
        if(start)
        {
            resetParameters();
            setCursor(cSelect);
            if (!searchMode) fo = new FactorOracle(fo.pattern());
        }
    }
    /** Genera el Grafo Oracle completo de pattern y lo pinta. */
    public void complete(){
        
        fo = new FactorOracle(fo.pattern());
        fo.Oracle();
        //ponemos a Null la Transicin current para que no la pinte
        currentTransition = null;
        currentNodeOracle = null;
    }
   /** Genera un paso del Grafo Oracle y lo pinta. 
     *  @return <b>true</b> si todava quedan ms pasos por realizar para acabar el grafo,
     *          <b>false</b> si el grafo est completo */
    public boolean step(){
        
        /*//Dejamos las cosas como estaban antes de hacer un nuevo paso
        if(currentTransition!=null) paintDefault();
        */
        int n = fo.getTotalTransitions();
        
        currentTransition = fo.step();
        // Si antes de llamar a go.step() haba menos transiciones es que se ha creado una nueva
        currentTransitionIsNew = (n < fo.getTotalTransitions());
        
        // Comprobamos si hemos avanzado al siguiente nodo
        if(fo.getCurrentNodeValue() != currentNodeOracle.getOrder() && 
           fo.getCurrentStep() != FactorOracleConst.STEP_0 && !fo.isEnded())
        currentNodeOracle = (NodeOracle)fo.getCurrentNode();
        
        /*paintSelected();
        strategy.show();*/
        
        numberOfSteps++;
        //if(this.fitToScreen) fitToScreen();
        return (fo.isEnded());
    }   
    /** Retrocede un paso del Grafo Oracle y lo pinta. 
     *  @return Nmero de paso actual. */
    public int stepBack(){
        
        if(numberOfSteps <= 1) reset(); //el reset tb pondr a 0 numberOfSteps
        else //tenemos algn paso al que retroceder diferente del inicio
        {
            //Reseteamos el GraphOracle
            fo = new FactorOracle(fo.pattern());
                       
            //Restamos el paso que hacemos atrs
            numberOfSteps--;
            for(int i=1; i< numberOfSteps; i++) fo.step();
                       
            //En el ltimo paso recogemos los datos necesarios para el mostrado
            int n = fo.getTotalTransitions();
            currentTransition = fo.step();
            // Si antes de llamar a go.step() haba menos transiciones es que se ha creado una nueva
            currentTransitionIsNew = (n < fo.getTotalTransitions());
        
            // En el paso 0 el nodo Actual es el siguiente del que queremos mostrar como actual
            if (fo.getCurrentStep() == FactorOracleConst.STEP_0)
                 currentNodeOracle = (NodeOracle)fo.getNode(fo.getCurrentNodeValue()-1);
            else currentNodeOracle = (NodeOracle)fo.getCurrentNode();
       }
       return numberOfSteps;
    }
    /** Aumenta en 1 el tamao de los nodos del grafo y lo reescala siempre que el
     *  tamao actual del grafo no supere los 100 pixels.
     *  @return La anchura del Node de PaintGraphOracle. */
    public int makeItBigger(){
        
        Dimension d = pfo.getNodeSize();
        
        resizeFactorOracle(new Dimension(d.width+1, d.height+1));
        return pfo.getNodeSize().width;
    }
    /** Disminuye en 1 el tamao de los nodos del grafo y lo reescala siempre que el
     *  tamao actual del grafo no sea inferios a 1 pixel. 
     *  @return La anchura del Node de PaintGraphOracle. */
    public int makeItSmaller(){
        
        Dimension d = pfo.getNodeSize();
        
        if(d.width>1) resizeFactorOracle(new Dimension(d.width-1, d.height-1));
        return pfo.getNodeSize().width;
    }
    /** Reescala el Grafo, calcula el nuevo GraphStartPoint y pinta el grafo.
     *  @param d Dimensin con la que se reescala el grafo. */
    public void resizeFactorOracle(Dimension d){
        
        int nodeHeightBefore = pfo.getNodeSize().height;
        pfo.resize(d);
         
        // Recalculamos el centro relativo a la posicin actual en el caso de que tengamos
        // que dibujar suffix-links: 
        // Coordenada x: La misma que tenamos.
        // Coordenada y: Nos colocamos en el centro (verticalmente hablando) del nodo antes de 
        //               hacer el resize (nodeHeightBefore/2) y luego nos movemos segn nuestra 
        //               altura para que el centro del grafo siga en el mismo lugar 
        //               (pgo.getNodeSize().height/2)
        if(drawLink){ 
            Point p = new Point(pfo.getGraphStartPoint().x, 
                          pfo.getGraphStartPoint().y + nodeHeightBefore/2 
                                                     - pfo.getNodeSize().height/2);
            
            pfo.setAutomatonStartPoint(p);
        }
    }
    /** Ajusta el tamao del grafo al de la ventana de forma horizontal y lo pinta.
     *  @return La anchura del Node de PaintGraphOracle. */
    public int fitToScreen(){
        
        //pgo.setGraphStartPoint(getInitialPoint());
        //lo que ocupa el nodo + el espacio de separacin entre nodos
        //int width = Math.max(1, (this.getWidth()- pgo.getGraphStartPoint().x)/ go.size());
        int width = Math.max(1, (this.getWidth()- getInitialPoint().x)/ fo.size());
        //Cogemos la separacin entre nodos
        int w = Math.max(Math.min(width * 7 /15, 100), 25);
        //Obtenemos el tamao del grafo
        width = Math.max(width-w, 1);
        //Redimensionamos el grafo calculando su nueva posicin de inicio
        resizeFactorOracle(new Dimension(width, width));
               
        return pfo.getNodeSize().width;
    }
    /** Mueve el punto de vista de la ventana hacia las direcciones que indiquen x e y,
     *  el nuevo punto de inicio ser PuntoActual.x + x, PuntoActual.y + y
     *  @param x Coordenada x que se aadir al punto de comienzo de dibujado del Grafo.
     *  @param y Coordenada y que se aadir al punto de comienzo de dibujado del Grafo.*/
    public void moveFromStartPoint(int x, int y){
        pfo.setAutomatonStartPoint(new Point(pfo.getGraphStartPoint().x + x, 
                                         pfo.getGraphStartPoint().y + y));
    }
    /** Mueve el punto de vista del Grafo a x e y.
     *  @param x Coordenada x que indica el punto de comienzo de dibujado del Grafo.
     *  @param y Coordenada y que indica el punto de comienzo de dibujado del Grafo.*/
    public void setFactorOracleStartPoint(int x, int y){
        pfo.setAutomatonStartPoint(new Point(x, y));
    }
    /** Mueve el punto de vista de la ventana al punto de vista inicial.*/
    public void moveToDefaultStartPoint(){
        pfo.setAutomatonStartPoint(getInitialPoint());
    }
    /** Pinta el Grafo Oracle actual, junto con el alfabeto que lo compone. */
        public void paint(Graphics g){
    
        if(!start)return;
        /* Ahora mismo no hacemos uso del Graphics pasado, si alguien quiere que su Graphics
         * sea usado habr de hacer uso de l */
        cleanBackground(pfo.getBackgroundColor());
        pfo.paintFactorOracle(g2d, fo, drawLink);
        
        //Pintamos lo que hubiera seleccionado, esto es necesario para
        //cuando hacemos reescalados ya que se llama a paintGraph.
        //Tenemos en cuenta el tipo de pintado segn searchMode
        if(searchMode) paintSelectedSearch();
        else paintSelected();
    fo.printPatternsFinals();
        //Aqu tendremos que pintar las etiquetas de los nodos finales
        if(fo.numberOfPatterns()>1)
        {
            //3 iguales
            if(fo.patternsFinals(0)==fo.patternsFinals(1) && 
               fo.patternsFinals(1)==fo.patternsFinals(2))
            {
                pfo.drawNodeTag(g2d, fo.patternsFinals(0), "1, 2, 3", true);
            }
            //2 iguales 1-2
            else if(fo.patternsFinals(0)==fo.patternsFinals(1))
            {
                pfo.drawNodeTag(g2d, fo.patternsFinals(0), "1, 2", true);
                pfo.drawNodeTag(g2d, fo.patternsFinals(2), "3", true);
            }
            //2 iguales 2-3
            else if(fo.patternsFinals(1)==fo.patternsFinals(2))
            {
                pfo.drawNodeTag(g2d, fo.patternsFinals(0), "1", true);
                pfo.drawNodeTag(g2d, fo.patternsFinals(1), "2, 3", true);
            }
            //3 diferentes
            else
            {
                pfo.drawNodeTag(g2d, fo.patternsFinals(0), "1", true);
                pfo.drawNodeTag(g2d, fo.patternsFinals(1), "2", true);
                pfo.drawNodeTag(g2d, fo.patternsFinals(2), "3", true);
            }
        }
                
        //pgo.drawGraphTitle(g, this.getSize(), go.pattern());
        strategy.show();  //Necesario para mostrar los cambios
        
        /*///////// SALVAR JPG //////////////////
        Dimension aux = this.getSize();
        Point aux2 = pgo.getGraphStartPoint();
        
        //Resize del canvas para pintar toda el grafo completo
        this.setSize(go.size() * (pgo.getNodeSize().width + pgo.getNodeSeparation()) + 100, 1024);
        //pgo.setGraphStartPoint(new Point(aux2.x, getSize().height/2 - pgo.getNodeSize().height/2));
        
        pgo.setGraphStartPoint(new Point(aux2.x, pgo.getNodeSize().height/2));
        
        
        //BufferedImage d = new BufferedImage(this.getWidth(),this.getHeight(), BufferedImage.TYPE_BYTE_INDEXED);
        Image d = createImage(this.getWidth(), this.getHeight());
        
        g = (Graphics2D)d.getGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING     , RenderingHints.VALUE_ANTIALIAS_ON);
        g.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING  , RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_DITHERING        , RenderingHints.VALUE_DITHER_ENABLE );
        g.setRenderingHint(RenderingHints.KEY_RENDERING        , RenderingHints.VALUE_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
        g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        
        //pgo.paintGraphOracle(g, go, true);
        pgo.paintGraphOracle(g, go, false);
        paintSelected();
        saveJPG(d, "testA_", ".jpg", new File(""));
        
        //dejamos todo como estaba
        this.setSize(aux);
        pgo.setGraphStartPoint(aux2);
        initGraphics2D();
        ////////////////////////////////////////*/
    }
    /** Pinta el Grafo Oracle actual con poco detalle. */
    public void paintRoughFactorOracle(){
    
        cleanBackground(pfo.getBackgroundColor());
        pfo.paintRoughFactorOracle(g2d, fo, drawLink);
        strategy.show();  //Necesario para mostrar los cambios
    }
    protected void initGraphics2D(){
        
        createBufferStrategy(2);
        strategy = getBufferStrategy();
        g2d = (Graphics2D)strategy.getDrawGraphics();
        //Le damos valores al render de g
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING     , RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING  , RenderingHints.VALUE_COLOR_RENDER_SPEED);
        g2d.setRenderingHint(RenderingHints.KEY_DITHERING        , RenderingHints.VALUE_DITHER_ENABLE );
        g2d.setRenderingHint(RenderingHints.KEY_RENDERING        , RenderingHints.VALUE_RENDER_SPEED);
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
    }
    //PRIVADAS
    protected void resetParameters(){
        
        leftButtonClicked = false;
        currentTransition = null; 
        currentTransitionIsNew = true;
        currentNodeOracle = new NodeOracle(-1);
        numberOfSteps = 0;
        if(searchMode) transitionsList.clear();
    }
    
    /** Devuelve el punto inicial por defecto donde debe estar el grafo relativo al 
     *  tamao de la ventana */
    private Point getInitialPoint(){
        
        if(drawLink) return new Point( 30, getY() + getHeight()/2 - pfo.getNodeSize().height/2);
        return new Point( 30, 15 );
    }
    /** Pinta la currentTransition, si no es nula y los nodos Origen y Destino
     *  de dicha Transition con los colores para nodos y transiciones seleccionadas
     *  del PaintGraphOracle pgo. */
    private void paintSelected(){
        
        //Si la transicin es nula es que GraphOracle est vaco
        if(currentTransition!=null && currentNodeOracle.getOrder()!=FactorOracleConst.NODE_EMPTY)
        {
            if(currentTransitionIsNew) //hemos creado una transicin/suffix-link (se pinta como creada)
            {
                if(currentTransition.getWord().equals("")) //Esto es un suffix-link
                                  pfo.drawCreatedLink(g2d, currentTransition); 
                else pfo.drawCreatedTransition(g2d, currentTransition, false); //Esto una transicin
            }
            else //estamos siguiendo una transicin/suffix-link ya existente (se pinta como seleccionada)
            {
                if(currentTransition.getWord().equals("")) 
                                  pfo.drawSelectedLink(g2d, currentTransition);
                else pfo.drawSelectedTransition(g2d, currentTransition, false);
            }
            // Pintamos los nodos iniciales y finales de la transicin actual
            pfo.paintSelectedNode(g2d, fo.getNode(currentTransition.getStart()));
            pfo.paintSelectedNode(g2d, fo.getNode(currentTransition.getEnd()));
            // Pintamos el nodo que estamos explorando actualmente
            pfo.paintCurrentNode(g2d, currentNodeOracle);
        }
    }
    /** Pinta las transiciones de transitionList */
    private void paintSelectedSearch(){
        
        if(transitionsList.size()>0)
        {
            Transition t = new Transition();
            //Si la transicin es nula es que GraphOracle est vaco
            for(int i=0; i <transitionsList.size(); i++)
            {
                t = (Transition)transitionsList.get(i);
                if(t.getWord().equals("")) pfo.drawSelectedLink(g2d, t);
                else pfo.drawSelectedTransition(g2d, t, false);
            
                pfo.paintSelectedNode(g2d, fo.getNode(t.getStart()));
            }
            pfo.paintCurrentNode(g2d, fo.getNode(t.getEnd()));
            //pgo.paintSelectedNode(g, go.getNode(t.getEnd()));
        }
        else pfo.paintCurrentNode(g2d, fo.getNode(0));
    }
    
    private void paintDefault(){
        
        //Si la transicin es nula es que GraphOracle est vaco
        if(currentTransition!=null)
        {
            //Comprobamos si es un suffix-link o una transicin lo que hay que pintar
            if(currentTransition.getWord().equals("")) 
                                  pfo.drawLink(g2d, currentTransition);
            else pfo.drawTransition(g2d, currentTransition, false);
            pfo.paintNode(g2d, fo.getNode(currentTransition.getStart()));
            pfo.paintNode(g2d, fo.getNode(currentTransition.getEnd()));
            pfo.paintNode(g2d, currentNodeOracle);
        }
    }

////////////TESTING, BORRAR AL FINAL ////////////////
    public static void paintBox(int x, int y, int w, int h, Graphics2D g){
       
       g.setColor(Color.black);
       //Caja + lineas
       g.drawRect(x, y, w, h);
       g.drawLine(x-(w/5), y+h/2, x+w+(w/5), y+h/2); //horizontal
       g.drawLine(x+w/2, y-(h/5), x+w/2, y+h+(h/5)); //vertical
   }
   
    //las imgenes que sean de 12x18 todas mejor
    public void paintIMG(int x, int y, Graphics2D g, String imageName, int num){
      
      try { 
      // note no leading slash
       //Image img = Toolkit.getDefaultToolkit().getImage("iconos/grafos/" + imageName);
       
       //Dibujamos el nodo
       Image img = ImageIO.read(getClass().getResource("iconos/grafos/" + imageName));
       g.drawImage(img, x, y, this);
       
       //Dibujamos el n en el interior del nodo
       int centerX = x + img.getWidth(this)/2;
       int centerY = y + img.getHeight(this)/2;
       
       //Si solo pintamos un carcter
       if (num<10)
       {    
            img = ImageIO.read(getClass().getResource("iconos/grafos/0" + num + ".gif"));
            g.drawImage(img, centerX - img.getWidth(this)/2, centerY - img.getHeight(this)/2, this);
       }
       //Si tenemos que pintar varios
       else if(num>=10)
       {
            img = ImageIO.read(getClass().getResource("iconos/grafos/0" + num/10 + ".gif"));
            Image img2 = ImageIO.read(getClass().getResource("iconos/grafos/0" + num%10 + ".gif"));
            
            int posX = centerX - (img.getWidth(this) + img2.getWidth(this))/2;
            int posY = centerY - img.getHeight(this)/2;
                    
            g.drawImage(img, posX, posY, this);                             
            g.drawImage(img2, posX + img.getWidth(this), centerY - img2.getHeight(this)/2, this); 
       }
      }
      catch (Exception ioe) {
      // Unable load a system image.
      System.out.println("No se pudo cargar la imagen [" + "" + "]");
      } 
    }
    
    public void saveJPG(Image img, String prefix, String suffix, File dir ){
        
        String s = null;
        BufferedImage bi = createBufferedImage(img);
        //BufferedImage b = makeBufferedImage(img, BufferedImage.TYPE_3BYTE_BGR);
        FileOutputStream out = null;
        try
        {
            File file = File.createTempFile(prefix, suffix, dir);
            s = file.getName();
            out = new FileOutputStream(file);
        }
        catch (java.io.FileNotFoundException io)
        { System.out.println("File Not Found"); }
        catch (java.io.IOException e) { System.err.println(e); }
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
        JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bi);
        try { encoder.encode(bi); }
        catch (java.io.IOException io) {
        System.out.println("IOException"); }
        try{ out.close(); }
        catch (java.io.IOException io) { System.out.println("IOException"); }
        System.out.println("JPEG Saved: " + s);
    }
    public BufferedImage createBufferedImage(Image image){
        if (image !=null)
        {
            BufferedImage bi = new BufferedImage(image.getWidth(null),image.getHeight(null),BufferedImage.TYPE_INT_RGB);
            Graphics g = bi.createGraphics();
            g.setColor(Color.white);
            g.fillRect(0, 0, image.getWidth(null), image.getHeight(null));
            g.drawImage(image, 0, 0, null);
            g.dispose();
            return bi;
        }
        else { return null; }
    }
/////////////////////////////////////////////////////

} 
