/*
 * Node.java
 *
 * Created on 18 de marzo de 2005, 15:11
 */

package oracle;

import java.util.ArrayList;

/**
 * 
 * @author Alejandro Touza Paredes
 */
public class Node {
    
    private int order;
    private String name;
    private ArrayList transitions;
    private int type;

    /** CONSTRUCTORAS */
    /** Crea una nueva instancia de Node */
    public Node() {
        
        this.order = 0;
        this.name = Integer.toString(this.order);
        this.type = FactorOracleConst.NODE_NORMAL;
        this.transitions = new ArrayList();
    }
    /**
     * Crea una nueva instancia de Node con el nombre que se le pasa
     * @param name 
     */
    public Node(int order){
        
        this.order = order;
        this.name = Integer.toString(this.order);
        this.type = FactorOracleConst.NODE_NORMAL;
        this.transitions = new ArrayList();
    }
    /** Crea una nueva instancia de Node con el Node que se le pasa*/
    public Node(Node n){
        
        this.transitions = new ArrayList();
        this.copy(n);
    }
    
    /** CONSULTORAS */
    /**
     * Retorna el nombre del Node
     * @return 
     */
    public int getOrder(){
        
        return this.order;
    }   
    /**
     * Retorna el nombre del Node
     * @return 
     */
    public String getName(){
        
        return this.name;
    }   
    /**
     * Retorna el tipo del Node
     * @return 
     */
    public int getType(){
        
        return this.type;
    }   
    /**
     * Retorna la Transition que hay en la posicin num PRIVADA????
     * @param num 
     * @return 
     */
    public Transition getTransition(int num){
      
      return (Transition)transitions.get(num);
    }    
    /**
     * Retorna el nmero de Transitions que tiene Node
     * @return 
     */   
    public int getNumberOfTransitions(){
        
        return transitions.size();
    }
    /** Retorna true si el name==-1 y el nmero de Transitions que tiene Node es 0*/   
    public boolean isEmpty(){
        
        return (order==FactorOracleConst.NODE_EMPTY && transitions.size()==0);
    }
    /**
     * Retorna true si existe una Transition como la pasada
     * @param t 
     * @return 
     */
    public boolean hasTransition(Transition t)
    {
        for(int i=0; i<transitions.size(); i++)
        {
            if (((Transition)(transitions.get(i))).isEquals(t)) return true;
        }
        return false;
    }
    /**
     * 
     * @param actual 
     * @return -1 si no existe la transicin o el n del nodo destino de la transicin.
     */
    public int hasTransition(String actual){
        
        for(int i=0; i<transitions.size(); i++)
        {
            Transition t = (Transition)(transitions.get(i));
            if (t.getWord().equals(actual)) return t.getEnd();
        }
        return -1; 
    }
        
    /** MODIFICADORAS */
    /**
     * 
     * @param name */
    public void setName(String name){
        
        this.name = name;
    }
    /**
     * 
     * @param name */
    public void setOrder(int order){
        
        this.order = order;
    }
    public void setNameLikeOrder(){
        
        this.name = Integer.toString(order);
    }
    /**
     * 
     * @param type */
    public void setType(int type){
        
        this.type = type;
    }
    /**
     * Aade la Transition t al Node
     * @param t 
     */
    public void putTransition(Transition t){
        
      this.transitions.add(t);
    }
    /**
     * Elimina la 1 Transition == t. Retorna cierto si existia alguna Transition t
     * @param t 
     * @return 
     */
    public boolean removeTransition(Transition t){
        
        for(int i=0; i<transitions.size(); i++)
        {
            if (((Transition)transitions.get(i)).isEquals(t))
            { 
                transitions.remove(i); 
                return true;
            }
        }
        return false;
    }
    /**
     * Realiza una copia del Node n en el Node que invoca la operacin
     * @param n 
     */
    public void copy(Node n){
        
        this.order = n.order;
        this.name = n.name;
        this.type = n.type;
        this.copyTransitions(n);
    }
    /**
     * Copia las transiciones del Node n al Node que invoca la operacin
     * @param n 
     */
    public void copyTransitions(Node n){
        
        for(int i=0; i<(n.transitions).size(); i++){
            
            this.transitions.add(i, n.transitions.get(i));
        }
    }
    
    /** VARIAS */
    public void print(){
        
        System.out.println("ooooooooooooooooooooooooooooooooooo");
        System.out.println("NODE <" + order + ">");
        System.out.println("## Transiciones ## ");
        for(int i=0; i<(transitions).size(); i++){
            
            System.out.println(((Transition)transitions.get(i)).print());
        }
    }


}
